@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: device_info.c,v 1.16.2.2 2005/03/28 19:15:19 david Exp $
+ * CVSID: $Id: device_info.c,v 1.27 2005/05/02 18:48:29 david Exp $
  *
  * device_store.c : Search for .fdi files and merge on match
  *
@@ -78,6 +78,12 @@ enum {
 
 	/** Processing an append element */
 	CURELEM_APPEND = 4,
+
+	/** Processing an prepend element */
+	CURELEM_PREPEND = 5,
+
+	/** Processing an prepend element */
+	CURELEM_REMOVE = 6
 };
 
 /** What and how to merge */
@@ -88,7 +94,9 @@ enum {
 	MERGE_TYPE_INT32,
 	MERGE_TYPE_UINT64,
 	MERGE_TYPE_DOUBLE,
-	MERGE_TYPE_COPY_PROPERTY
+	MERGE_TYPE_COPY_PROPERTY,
+	MERGE_TYPE_STRLIST,
+	MERGE_TYPE_REMOVE
 };
 
 /** Parsing Context
@@ -158,7 +166,7 @@ typedef struct {
  *  @param  prop_result_size    Size of property string
  *  @return                     TRUE if and only if the path resolved.
  */
-gboolean
+static gboolean
 resolve_udiprop_path (const char *path, const char *source_udi,
 		      char *udi_result, size_t udi_result_size, 
 		      char *prop_result, size_t prop_result_size)
@@ -270,29 +278,29 @@ match_compare_property (HalDevice *d, const char *key, const char *right_side, d
 
 	proptype = hal_device_property_get_type (d, key);
 	switch (proptype) {
-	case DBUS_TYPE_STRING:
+	case HAL_PROPERTY_TYPE_STRING:
 		*result = (dbus_int64_t) strcmp (hal_device_property_get_string (d, key), right_side);
 		rc = TRUE;
 		break;
 
-	case DBUS_TYPE_INT32:
+	case HAL_PROPERTY_TYPE_INT32:
 		*result = ((dbus_int64_t) hal_device_property_get_int (d, key)) - strtoll (right_side, NULL, 0);
 		rc = TRUE;
 		break;
 
-	case DBUS_TYPE_UINT64:
+	case HAL_PROPERTY_TYPE_UINT64:
 		*result = ((dbus_int64_t) hal_device_property_get_uint64 (d, key)) - ((dbus_int64_t) strtoll (right_side, NULL, 0));
 		rc = TRUE;
 		break;
 
-	case DBUS_TYPE_DOUBLE:
+	case HAL_PROPERTY_TYPE_DOUBLE:
 		*result = (dbus_int64_t) ceil (hal_device_property_get_double (d, key) - atof (right_side));
 		rc = TRUE;
 		break;
 
 	default:
 		/* explicit fallthrough */
-	case DBUS_TYPE_BOOLEAN:
+	case HAL_PROPERTY_TYPE_BOOLEAN:
 		/* explicit blank since this doesn't make sense */
 		break;
 	}
@@ -355,7 +363,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		/*HAL_INFO(("Checking that key='%s' is a string that "
 		  "equals '%s'", key, value)); */
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_STRING)
 			return FALSE;
 
 		if (strcmp (hal_device_property_get_string (d, prop_to_check),
@@ -375,7 +383,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		/*HAL_INFO (("Checking that key='%s' is a int that equals %d", 
 		  key, value));*/
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_INT32)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_INT32)
 			return FALSE;
 
 		if (hal_device_property_get_int (d, prop_to_check) != value) {
@@ -394,7 +402,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		/*HAL_INFO (("Checking that key='%s' is a int that equals %d", 
 		  key, value));*/
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_UINT64)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_UINT64)
 			return FALSE;
 
 		if (hal_device_property_get_uint64 (d, prop_to_check) != value) {
@@ -418,7 +426,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		  key, value ? "TRUE" : "FALSE"));*/
 
 		if (hal_device_property_get_type (d, prop_to_check) != 
-		    DBUS_TYPE_BOOLEAN)
+		    HAL_PROPERTY_TYPE_BOOLEAN)
 			return FALSE;
 
 		if (hal_device_property_get_bool (d, prop_to_check) != value)
@@ -450,7 +458,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		if (strcmp (attr[3], "false") == 0)
 			should_be_empty = FALSE;
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_STRING)
 			return FALSE;
 
 		if (hal_device_has_property (d, prop_to_check))
@@ -477,7 +485,7 @@ handle_match (ParsingContext * pc, const char **attr)
 		if (strcmp (attr[3], "false") == 0)
 			should_be_ascii = FALSE;
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_STRING)
 			return FALSE;
 
 		is_ascii = TRUE;
@@ -509,7 +517,7 @@ handle_match (ParsingContext * pc, const char **attr)
 
 		/*HAL_INFO (("d->udi='%s', prop_to_check='%s'", d->udi, prop_to_check));*/
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
+		if (hal_device_property_get_type (d, prop_to_check) != HAL_PROPERTY_TYPE_STRING)
 			return FALSE;
 
 		if (hal_device_has_property (d, prop_to_check)) {
@@ -538,18 +546,33 @@ handle_match (ParsingContext * pc, const char **attr)
 
 		needle = attr[3];
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
-			return FALSE;
-
-		if (hal_device_has_property (d, prop_to_check)) {
-			const char *haystack;
-
-			haystack = hal_device_property_get_string (d, prop_to_check);
-			if (needle != NULL && haystack != NULL && strstr (haystack, needle)) {
-				contains = TRUE;
+		if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_STRING) {
+			if (hal_device_has_property (d, prop_to_check)) {
+				const char *haystack;
+				
+				haystack = hal_device_property_get_string (d, prop_to_check);
+				if (needle != NULL && haystack != NULL && strstr (haystack, needle)) {
+					contains = TRUE;
+				}
+				
 			}
+		} else if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_STRLIST && 
+			   needle != NULL) {
+			GSList *i;
+			GSList *value;
 
+			value = hal_device_property_get_strlist (d, prop_to_check);
+			for (i = value; i != NULL; i = g_slist_next (i)) {
+				const char *str = i->data;
+				if (strcmp (str, needle) == 0) {
+					contains = TRUE;
+					break;
+				}
+			}
+		} else {
+			return FALSE;
 		}
+
 		return contains;
 	} else if (strcmp (attr[2], "contains_ncase") == 0) {
 		const char *needle;
@@ -557,22 +580,37 @@ handle_match (ParsingContext * pc, const char **attr)
 
 		needle = attr[3];
 
-		if (hal_device_property_get_type (d, prop_to_check) != DBUS_TYPE_STRING)
-			return FALSE;
-
-		if (hal_device_has_property (d, prop_to_check)) {
-			char *needle_lowercase;
-			char *haystack_lowercase;
-
-			needle_lowercase   = g_utf8_strdown (needle, -1);
-			haystack_lowercase = g_utf8_strdown (hal_device_property_get_string (d, prop_to_check), -1);
-			if (needle_lowercase != NULL && haystack_lowercase != NULL && strstr (haystack_lowercase, needle_lowercase)) {
-				contains_ncase = TRUE;
+		if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_STRING) {
+			if (hal_device_has_property (d, prop_to_check)) {
+				char *needle_lowercase;
+				char *haystack_lowercase;
+				
+				needle_lowercase   = g_utf8_strdown (needle, -1);
+				haystack_lowercase = g_utf8_strdown (hal_device_property_get_string (d, prop_to_check), -1);
+				if (needle_lowercase != NULL && haystack_lowercase != NULL && strstr (haystack_lowercase, needle_lowercase)) {
+					contains_ncase = TRUE;
+				}
+				
+				g_free (needle_lowercase);
+				g_free (haystack_lowercase);
 			}
+		} else if (hal_device_property_get_type (d, prop_to_check) == HAL_PROPERTY_TYPE_STRLIST && 
+			   needle != NULL) {
+			GSList *i;
+			GSList *value;
 
-			g_free (needle_lowercase);
-			g_free (haystack_lowercase);
+			value = hal_device_property_get_strlist (d, prop_to_check);
+			for (i = value; i != NULL; i = g_slist_next (i)) {
+				const char *str = i->data;
+				if (g_ascii_strcasecmp (str, needle) == 0) {
+					contains_ncase = TRUE;
+					break;
+				}
+			}
+		} else {
+			return FALSE;
 		}
+
 		return contains_ncase;
 	} else if (strcmp (attr[2], "compare_lt") == 0) {
 		dbus_int64_t result;
@@ -652,6 +690,10 @@ handle_merge (ParsingContext * pc, const char **attr)
 		/* match string property */
 		pc->merge_type = MERGE_TYPE_DOUBLE;
 		return;
+	} else if (strcmp (attr[3], "strlist") == 0) {
+		/* match string property */
+		pc->merge_type = MERGE_TYPE_STRLIST;
+		return;
 	} else if (strcmp (attr[3], "copy_property") == 0) {
 		/* copy another property */
 		pc->merge_type = MERGE_TYPE_COPY_PROPERTY;
@@ -661,13 +703,13 @@ handle_merge (ParsingContext * pc, const char **attr)
 	return;
 }
 
-/** Called when the append element begins.
+/** Called when the append or prepend element begins.
  *
  *  @param  pc                  Parsing context
  *  @param  attr                Attribute key/value pairs
  */
 static void
-handle_append (ParsingContext * pc, const char **attr)
+handle_append_prepend (ParsingContext * pc, const char **attr)
 {
 	int num_attrib;
 
@@ -688,9 +730,13 @@ handle_append (ParsingContext * pc, const char **attr)
 		return;
 
 	if (strcmp (attr[3], "string") == 0) {
-		/* match string property */
+		/* append to a string */
 		pc->merge_type = MERGE_TYPE_STRING;
 		return;
+	} else if (strcmp (attr[3], "strlist") == 0) {
+		/* append to a string list*/
+		pc->merge_type = MERGE_TYPE_STRLIST;
+		return;
 	} else if (strcmp (attr[3], "copy_property") == 0) {
 		/* copy another property */
 		pc->merge_type = MERGE_TYPE_COPY_PROPERTY;
@@ -700,6 +746,48 @@ handle_append (ParsingContext * pc, const char **attr)
 	return;
 }
 
+/** Called when the append or prepend element begins.
+ *
+ *  @param  pc                  Parsing context
+ *  @param  attr                Attribute key/value pairs
+ */
+static void
+handle_remove (ParsingContext * pc, const char **attr)
+{
+	int num_attrib;
+
+	pc->merge_type = MERGE_TYPE_UNKNOWN;
+
+	for (num_attrib = 0; attr[num_attrib] != NULL; num_attrib++) {
+		;
+	}
+
+	if (num_attrib != 2 && num_attrib != 4)
+		return;
+
+	if (strcmp (attr[0], "key") != 0)
+		return;
+	strncpy (pc->merge_key, attr[1], MAX_KEY_SIZE);
+
+	if (num_attrib == 4) {
+		if (strcmp (attr[2], "type") != 0)
+			return;
+
+		if (strcmp (attr[3], "strlist") == 0) {
+			/* remove from strlist */
+			pc->merge_type = MERGE_TYPE_STRLIST;
+			return;
+		} else {
+			pc->merge_type = MERGE_TYPE_UNKNOWN;
+			return;
+		}
+	} else {
+		pc->merge_type = MERGE_TYPE_REMOVE;
+	}
+
+	return;
+}
+
 /** Abort parsing of document
  *
  *  @param  pc                  Parsing context
@@ -790,7 +878,41 @@ start (ParsingContext * pc, const char *el, const char **attr)
 
 		pc->curelem = CURELEM_APPEND;
 		if (pc->match_ok) {
-			handle_append (pc, attr);
+			handle_append_prepend (pc, attr);
+		} else {
+			/*HAL_INFO(("No merge!")); */
+		}
+	} else if (strcmp (el, "prepend") == 0) {
+		if (pc->curelem != CURELEM_DEVICE
+		    && pc->curelem != CURELEM_MATCH) {
+			HAL_ERROR (("%s:%d:%d: Element <prepend> can only be "
+				    "inside <device> and <match>", 
+				    pc->file, 
+				    XML_GetCurrentLineNumber (pc->parser), 
+				    XML_GetCurrentColumnNumber (pc->parser)));
+			parsing_abort (pc);
+		}
+
+		pc->curelem = CURELEM_PREPEND;
+		if (pc->match_ok) {
+			handle_append_prepend (pc, attr);
+		} else {
+			/*HAL_INFO(("No merge!")); */
+		}
+	} else if (strcmp (el, "remove") == 0) {
+		if (pc->curelem != CURELEM_DEVICE
+		    && pc->curelem != CURELEM_MATCH) {
+			HAL_ERROR (("%s:%d:%d: Element <remove> can only be "
+				    "inside <device> and <match>", 
+				    pc->file, 
+				    XML_GetCurrentLineNumber (pc->parser), 
+				    XML_GetCurrentColumnNumber (pc->parser)));
+			parsing_abort (pc);
+		}
+
+		pc->curelem = CURELEM_REMOVE;
+		if (pc->match_ok) {
+			handle_remove (pc, attr);
 		} else {
 			/*HAL_INFO(("No merge!")); */
 		}
@@ -853,10 +975,18 @@ end (ParsingContext * pc, const char *el)
 
 		switch (pc->merge_type) {
 		case MERGE_TYPE_STRING:
-			hal_device_property_set_string (pc->device, pc->merge_key,
-						pc->cdata_buf);
+			hal_device_property_set_string (pc->device, pc->merge_key, pc->cdata_buf);
 			break;
 
+		case MERGE_TYPE_STRLIST:
+		{
+			int type = hal_device_property_get_type (pc->device, pc->merge_key);
+			if (type == HAL_PROPERTY_TYPE_STRLIST || type == HAL_PROPERTY_TYPE_INVALID) {
+				hal_device_property_remove (pc->device, pc->merge_key);
+				hal_device_property_strlist_append (pc->device, pc->merge_key, pc->cdata_buf);
+			}
+			break;
+		}
 
 		case MERGE_TYPE_INT32:
 			{
@@ -933,34 +1063,96 @@ end (ParsingContext * pc, const char *el)
 			break;
 		}
 	} else if (pc->curelem == CURELEM_APPEND && pc->match_ok && 
-		   hal_device_property_get_type (pc->device, pc->merge_key) == DBUS_TYPE_STRING) {
+		   (hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_STRING ||
+		    hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_STRLIST ||
+		    hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_INVALID)) {
 		char buf[256];
 		char buf2[256];
 
 		/* As soon as we are appending, we have matched the device... */
 		pc->device_matched = TRUE;
 
-		switch (pc->merge_type) {
-		case MERGE_TYPE_STRING:
-			strncpy (buf, pc->cdata_buf, sizeof (buf));
-			break;
-
-		case MERGE_TYPE_COPY_PROPERTY:
-			hal_device_property_get_as_string (pc->device, pc->cdata_buf, buf, sizeof (buf));
-			break;
-
-		default:
-			HAL_ERROR (("Unknown merge_type=%d='%c'",
-				    pc->merge_type, pc->merge_type));
-			break;
+		if (pc->merge_type == MERGE_TYPE_STRLIST) {
+			hal_device_property_strlist_append (pc->device, pc->merge_key, pc->cdata_buf);
+		} else {
+			const char *existing_string;
+			
+			switch (pc->merge_type) {
+			case MERGE_TYPE_STRING:
+				strncpy (buf, pc->cdata_buf, sizeof (buf));
+				break;
+				
+			case MERGE_TYPE_COPY_PROPERTY:
+				hal_device_property_get_as_string (pc->device, pc->cdata_buf, buf, sizeof (buf));
+				break;
+				
+			default:
+				HAL_ERROR (("Unknown merge_type=%d='%c'", pc->merge_type, pc->merge_type));
+				break;
+			}
+			
+			existing_string = hal_device_property_get_string (pc->device, pc->merge_key);
+			if (existing_string != NULL) {
+				strncpy (buf2, existing_string, sizeof (buf2));
+				strncat (buf2, buf, sizeof (buf2) - strlen(buf2));
+			} else {
+				strncpy (buf2, buf, sizeof (buf2));
+			}
+			hal_device_property_set_string (pc->device, pc->merge_key, buf2);
 		}
+	} else if (pc->curelem == CURELEM_PREPEND && pc->match_ok && 
+		   (hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_STRING ||
+		    hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_STRLIST ||
+		    hal_device_property_get_type (pc->device, pc->merge_key) == HAL_PROPERTY_TYPE_INVALID)) {
+		char buf[256];
+		char buf2[256];
 
-		strncpy (buf2, hal_device_property_get_string (pc->device, pc->merge_key), sizeof (buf2));
-		strncat (buf2, buf, sizeof (buf2) - strlen(buf2));
-		hal_device_property_set_string (pc->device, pc->merge_key, buf2);
+		/* As soon as we are prepending, we have matched the device... */
+		pc->device_matched = TRUE;
+
+		if (pc->merge_type == MERGE_TYPE_STRLIST) {
+			hal_device_property_strlist_prepend (pc->device, pc->merge_key, pc->cdata_buf);
+		} else {
+			const char *existing_string;
+			
+			switch (pc->merge_type) {
+			case MERGE_TYPE_STRING:
+				strncpy (buf, pc->cdata_buf, sizeof (buf));
+				break;
+				
+			case MERGE_TYPE_COPY_PROPERTY:
+				hal_device_property_get_as_string (pc->device, pc->cdata_buf, buf, sizeof (buf));
+				break;
+				
+			default:
+				HAL_ERROR (("Unknown merge_type=%d='%c'", pc->merge_type, pc->merge_type));
+				break;
+			}
+			
+			existing_string = hal_device_property_get_string (pc->device, pc->merge_key);
+			if (existing_string != NULL) {
+				strncpy (buf2, buf, sizeof (buf2));
+				strncat (buf2, existing_string, sizeof (buf2) - strlen(buf2));
+			} else {
+				strncpy (buf2, buf, sizeof (buf2));
+			}
+			hal_device_property_set_string (pc->device, pc->merge_key, buf2);
+		}
+	} else if (pc->curelem == CURELEM_REMOVE && pc->match_ok) {
+
+		if (pc->merge_type == MERGE_TYPE_STRLIST) {
+			/* covers <remove key="foobar" type="strlist">blah</remove> */
+			hal_device_property_strlist_remove (pc->device, pc->merge_key, pc->cdata_buf);
+		} else {
+			/* only allow <remove key="foobar"/>, not <remove key="foobar">blah</remove> */
+			if (strlen (pc->cdata_buf) == 0) {
+				hal_device_property_remove (pc->device, pc->merge_key);
+			}
+		}
 	}
 
 
+
 	pc->cdata_buf_len = 0;
 	pc->depth--;
 
@@ -1035,7 +1227,7 @@ process_fdi_file (const char *dir, const char *filename,
 
 	snprintf (buf, sizeof (buf), "%s/%s", dir, filename);
 
-	/*HAL_INFO(("analysing file %s", buf)); */
+	/*HAL_INFO(("analyzing file %s", buf));*/
 
 	/* open file and read it into a buffer; it's a small file... */
 	file = fopen (buf, "r");
@@ -1132,7 +1324,7 @@ my_alphasort(const void *a, const void *b)
  *  @param  d                   Device to merge information into
  *  @return                     #TRUE if information was merged
  */
-static int
+static dbus_bool_t
 scan_fdi_files (const char *dir, HalDevice * d)
 {
 	int i;
@@ -1172,7 +1364,7 @@ scan_fdi_files (const char *dir, HalDevice * d)
 			    filename[len - 3] == 'f' &&
 			    filename[len - 2] == 'd' &&
 			    filename[len - 1] == 'i') {
-				HAL_INFO (("scan_fdi_files: Processing file '%s'", filename));
+				/*HAL_INFO (("scan_fdi_files: Processing file '%s'", filename));*/
 				found_fdi_file = process_fdi_file (dir, filename, d);
 				if (found_fdi_file) {
 					HAL_INFO (("*** Matched file %s/%s", dir, filename));
@@ -1222,7 +1414,6 @@ scan_fdi_files (const char *dir, HalDevice * d)
 	return found_fdi_file;
 }
 
-
 /** Search the device info file repository for a .fdi file to merge
  *  more information into the device object.
  *
@@ -1230,10 +1421,69 @@ scan_fdi_files (const char *dir, HalDevice * d)
  *  @return                     #TRUE if information was merged
  */
 dbus_bool_t
-di_search_and_merge (HalDevice *d)
+di_search_and_merge (HalDevice *d, DeviceInfoType type)
 {
+	static gboolean have_checked_hal_fdi_source = FALSE;
+	static char *hal_fdi_source_preprobe = NULL;
+	static char *hal_fdi_source_information = NULL;
+	static char *hal_fdi_source_policy = NULL;
+	dbus_bool_t ret;
+	char *s1;
+	char *s2;
 
-	return scan_fdi_files (PACKAGE_DATA_DIR "/hal/fdi", d);
+	ret = FALSE;
+
+	if (!have_checked_hal_fdi_source) {
+		hal_fdi_source_preprobe    = getenv ("HAL_FDI_SOURCE_PREPROBE");
+		hal_fdi_source_information = getenv ("HAL_FDI_SOURCE_INFORMATION");
+		hal_fdi_source_policy      = getenv ("HAL_FDI_SOURCE_POLICY");
+		have_checked_hal_fdi_source = TRUE;
+	}
+
+	switch (type) {
+	case DEVICE_INFO_TYPE_PREPROBE:
+		if (hal_fdi_source_preprobe != NULL) {
+			s1 = hal_fdi_source_preprobe;
+			s2 = NULL;
+		} else {
+			s1 = PACKAGE_DATA_DIR "/hal/fdi/preprobe";
+			s2 = PACKAGE_SYSCONF_DIR "/hal/fdi/preprobe";
+		}
+		break;
+
+	case DEVICE_INFO_TYPE_INFORMATION:
+		if (hal_fdi_source_information != NULL) {
+			s1 = hal_fdi_source_information;
+			s2 = NULL;
+		} else {
+			s1 = PACKAGE_DATA_DIR "/hal/fdi/information";
+			s2 = PACKAGE_SYSCONF_DIR "/hal/fdi/information";
+		}
+		break;
+
+	case DEVICE_INFO_TYPE_POLICY:
+		if (hal_fdi_source_policy != NULL) {
+			s1 = hal_fdi_source_policy;
+			s2 = NULL;
+		} else {
+			s1 = PACKAGE_DATA_DIR "/hal/fdi/policy";
+			s2 = PACKAGE_SYSCONF_DIR "/hal/fdi/policy";
+		}
+		break;
+
+	default:
+		s1 = NULL;
+		s2 = NULL;
+		HAL_ERROR (("Bogus device information type %d", type));
+		break;
+	}
+
+	if (s1 != NULL)
+		ret = scan_fdi_files (s1, d) || ret;
+	if (s2 != NULL)
+		ret = scan_fdi_files (s2, d) || ret;
+
+	return ret;
 }
 
 /** @} */
