@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: device_info.h,v 1.3 2004/04/03 07:46:33 david Exp $
+ * CVSID: $Id: device_info.h,v 1.4 2005/02/24 16:50:34 david Exp $
  *
  * device_store.c : Search for .fdi files and merge on match
  *
@@ -32,6 +32,12 @@
 
 #include "device_store.h"
 
-dbus_bool_t di_search_and_merge (HalDevice *d);
+typedef enum {
+	DEVICE_INFO_TYPE_PREPROBE,
+	DEVICE_INFO_TYPE_INFORMATION,
+	DEVICE_INFO_TYPE_POLICY
+} DeviceInfoType;
+
+dbus_bool_t di_search_and_merge (HalDevice *d, DeviceInfoType type);
 
 #endif				/* DEVICE_INFO_H */
