@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: hald.c,v 1.21 2004/10/18 22:52:05 david Exp $
+ * CVSID: $Id: hald.c,v 1.37 2005/06/23 14:50:04 david Exp $
  *
  * hald.c : main startup for HAL daemon
  *
@@ -38,26 +38,26 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
-#include <sys/prctl.h>
-#include <sys/capability.h>
 #include <grp.h>
 #include <syslog.h>
 
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+/*#include "master_slave.h"*/
 
-#include "callout.h"
 #include "logger.h"
 #include "hald.h"
 #include "device_store.h"
-#include "pstore.h"
 #include "device_info.h"
 #include "osspec.h"
 #include "hald_dbus.h"
-#include "hald_conf.h"
+#include "util.h"
 
-static void delete_pid(void) {
-    unlink(HALD_PID_FILE);
+static void delete_pid(void)
+{
+	unlink(HALD_PID_FILE);
 }
 
 /**
@@ -70,18 +70,69 @@ static HalDeviceStore *global_device_list = NULL;
 
 static HalDeviceStore *temporary_device_list = NULL;
 
-static HalPStore *pstore_sys = NULL;
+static GSList *running_addons = NULL;
+
+static void 
+addon_terminated (HalDevice *d, gboolean timed_out, gint return_code, 
+		  gpointer data1, gpointer data2, HalHelperData *helper_data)
+{
+	running_addons = g_slist_remove (running_addons, helper_data);
+}
 
 static void
 gdl_store_changed (HalDeviceStore *store, HalDevice *device,
 		   gboolean is_added, gpointer user_data)
 {
-	if (is_added)
-		HAL_INFO (("Added device to GDL; udi=%s",
-			   hal_device_get_udi(device)));
-	else
-		HAL_INFO (("Removed device from GDL; udi=%s",
-			   hal_device_get_udi(device)));
+	if (is_added) {
+		GSList *addons;
+
+		HAL_INFO (("Added device to GDL; udi=%s", hal_device_get_udi(device)));
+
+		if ((addons = hal_device_property_get_strlist (device, "info.addons")) != NULL) {
+			GSList *i;
+			
+			for (i = addons; i != NULL; i = g_slist_next (i)) {
+				const gchar *command_line;
+				HalHelperData *helper_data;
+				gchar *extra_env[2] = {"HALD_ACTION=addon", NULL};
+
+				command_line = (const gchar *) i->data;
+				helper_data = hal_util_helper_invoke (command_line, extra_env, device,
+								      NULL, NULL,
+								      addon_terminated, 0 /* no timeout */);
+
+				if (helper_data != NULL) {
+					HAL_INFO (("Invoked addon %s with pid %d for udi %s", 
+						   command_line, helper_data->pid, helper_data->d->udi));
+					running_addons = g_slist_prepend (running_addons, helper_data);
+				}
+			}
+		}
+	} else {
+		GSList *i;
+
+		HAL_INFO (("Removed device from GDL; udi=%s", hal_device_get_udi(device)));
+
+	start_from_beginning:
+
+		/* may have several addons running */
+		for (i = running_addons; i != NULL; i = g_slist_next (i)) {
+			HalHelperData *helper_data;
+
+			helper_data = (HalHelperData *) (i->data);
+			if (helper_data->d == device) {
+				HAL_INFO (("Terminating addon with pid %d for udi %s", 
+					   helper_data->pid, helper_data->d->udi));
+				/* will force a callback - the callback removes us from the list */
+				hal_util_terminate_helper (helper_data);
+				/* TODO: is it safe to remove an elem from a GSList and keep iterating? 
+				 *       Better play it safe for now.
+				 */
+				goto start_from_beginning;
+			}
+		}
+		
+	}
 
 	/*hal_device_print (device);*/
 
@@ -100,7 +151,7 @@ gdl_property_changed (HalDeviceStore *store, HalDevice *device,
 
 	/* only execute the callouts if the property _changed_ */
 	if (added == FALSE && removed == FALSE)
-		hal_callout_property (device, key);
+		/*hal_callout_property (device, key)*/;
 }
 
 static void
@@ -108,7 +159,7 @@ gdl_capability_added (HalDeviceStore *store, HalDevice *device,
 		      const char *capability, gpointer user_data)
 {
 	manager_send_signal_new_capability (device, capability);
-	hal_callout_capability (device, capability, TRUE);
+	/*hal_callout_capability (device, capability, TRUE)*/;
 }
 
 HalDeviceStore *
@@ -142,16 +193,6 @@ hald_get_tdl (void)
 	return temporary_device_list;
 }
 
-HalPStore *
-hald_get_pstore_sys (void)
-{
-	if (pstore_sys == NULL)
-		pstore_sys = hal_pstore_open (PACKAGE_LOCALSTATEDIR
-					      "/lib/hal");
-
-	return pstore_sys;
-}
-
 /**
  * @defgroup MainDaemon Basic functions
  * @ingroup HalDaemon
@@ -168,12 +209,11 @@ usage ()
 	fprintf (stderr, "\n" "usage : hald [--daemon=yes|no] [--verbose=yes|no] [--help]\n");
 	fprintf (stderr,
 		 "\n"
-		 "        --daemon=yes|no    Become a daemon\n"
-		 "        --verbose=yes|no   Print out debug (overrides HALD_VERBOSE)\n"
- 		 "        --drop-privileges  Run as normal user instead of root (calling of\n"
- 		 "                           external scripts to modify fstab etc. will not work\n" 
-		 "                           run as root)\n"
-		 "        --help             Show this information and exit\n"
+		 "        --daemon=yes|no      Become a daemon\n"
+		 "        --verbose=yes|no     Print out debug (overrides HALD_VERBOSE)\n"
+ 		 "        --retain-privileges  Run as root instead of normal user (calling of\n"
+ 		 "                             external scripts to modify fstab etc. will work)\n" 
+		 "        --help               Show this information and exit\n"
 		 "\n"
 		 "The HAL daemon detects devices present in the system and provides the\n"
 		 "org.freedesktop.Hal service through the system-wide message bus provided\n"
@@ -214,49 +254,38 @@ sigterm_iochn_data (GIOChannel *source,
 	GError *err = NULL;
 	gchar data[1];
 	gsize bytes_read;
+	unsigned int num_helpers;
 
 	/* Empty the pipe */
 	if (G_IO_STATUS_NORMAL != 
 	    g_io_channel_read_chars (source, data, 1, &bytes_read, &err)) {
-		HAL_ERROR (("Error emptying callout notify pipe: %s",
+		HAL_ERROR (("Error emptying sigterm pipe: %s",
 				   err->message));
 		g_error_free (err);
 		goto out;
 	}
 
-	fprintf (stderr, "SIGTERM, initiating shutdown");
-
-	hald_is_shutting_down = TRUE;
-
-	osspec_shutdown();
+	HAL_INFO (("Caught SIGTERM, initiating shutdown"));
+	num_helpers = hal_util_kill_all_helpers ();
+	HAL_INFO (("Killed %d helpers; exiting"));
+	exit (0);
 
 out:
 	return TRUE;
 }
 
-void 
-osspec_shutdown_done (void)
-{
-	exit (0);
-}
-
 
 /** This is set to #TRUE if we are probing and #FALSE otherwise */
 dbus_bool_t hald_is_initialising;
 
-/** This is set to #TRUE if we are shutting down and #FALSE otherwise */
-dbus_bool_t hald_is_shutting_down;
-
 static int startup_daemonize_pipe[2];
 
 /** Drop all but necessary privileges from hald when it runs as root.  Set the
- *  running user id to HAL_USER and group to HAL_GROUP and grant the following 
- *  capabilities: CAP_NET_ADMIN
+ *  running user id to HAL_USER and group to HAL_GROUP
  */
 static void
 drop_privileges ()
 {
-    cap_t cap;
     struct passwd *pw = NULL;
     struct group *gr = NULL;
 
@@ -274,12 +303,6 @@ drop_privileges ()
 	exit (-1);
     }
 
-    /* keep capabilities and change uid/gid */
-    if( prctl (PR_SET_KEEPCAPS, 1, 0, 0, 0)) {
-	HAL_ERROR (("drop_privileges: could not keep capabilities"));
-	exit (-1);
-    }
-
     if( initgroups (HAL_USER, gr->gr_gid)) {
 	HAL_ERROR (("drop_privileges: could not initialize groups"));
 	exit (-1);
@@ -294,23 +317,75 @@ drop_privileges ()
 	HAL_ERROR (("drop_privileges: could not set user id"));
 	exit (-1);
     }
-
-    /* only keep necessary capabilities */
-    cap = cap_from_text ("cap_net_admin=ep");
-
-    if(cap_set_proc(cap)) {
-	HAL_WARNING (("Your kernel does not support capabilities; some features will not be available."));
-	/* we do not fail on kernels which do not support capabilities, since
-	 * only very few features actually depend on them */
-    }
-
-    if(cap_free (cap)) {
-	HAL_ERROR (("drop_privileges: cap_free"));
-	exit (-1);
-    }
 }
 
 
+/*--------------------------------------------------------------------------------------------------*/
+
+static gboolean child_died = FALSE;
+
+static void 
+handle_sigchld (int value)
+{
+	child_died = TRUE;
+}
+
+static int 
+parent_wait_for_child (int child_fd, pid_t child_pid)
+{
+	fd_set rfds;
+	fd_set efds;
+	struct timeval tv;
+	int retval;
+	int ret;
+
+	signal(SIGCHLD, handle_sigchld);
+
+	/* wait for either
+	 *
+	 * o Child writes something to the child_fd; means that device
+	 *   probing is completed and the parent should exit with success
+	 *
+	 * o Child is killed (segfault etc.); means that parent should exit
+	 *   with failure
+	 *
+	 * o Timeout; means that we should kill the child and exit with
+	 *   failure
+	 *
+	 */
+
+	FD_ZERO(&rfds);
+	FD_SET(child_fd, &rfds);
+	FD_ZERO(&efds);
+	FD_SET(child_fd, &efds);
+	/* Wait up to 25 seconds for device probing */
+	tv.tv_sec = 25;
+	tv.tv_usec = 0;
+
+	retval = select (child_fd + 1, &rfds, NULL, &efds, &tv);
+
+	if (child_died) {
+		/* written from handle_sigchld */
+		ret = 1;
+		goto out;
+	}
+
+	if (retval > 0) {
+		/* means child wrote to socket or closed it; all good */
+		ret = 0;
+		goto out;
+	}
+
+	/* assume timeout; kill child */
+	kill (child_pid, SIGTERM);
+	ret = 2;
+
+out:
+	return ret;
+}
+
+/*--------------------------------------------------------------------------------------------------*/
+
 /** Entry point for HAL daemon
  *
  *  @param  argc                Number of arguments
@@ -322,6 +397,12 @@ main (int argc, char *argv[])
 {
 	GMainLoop *loop;
 	guint sigterm_iochn_listener_source_id;
+	gboolean retain_privs;
+	char *path;
+	char newpath[512];
+
+	retain_privs = FALSE;
+  
 
 	openlog ("hald", LOG_PID, LOG_DAEMON);
 
@@ -333,6 +414,18 @@ main (int argc, char *argv[])
 	else
 		hald_is_verbose = FALSE;
 
+	/* our helpers are installed into libexec, so adjust out $PATH
+	 * to include this at the end (since we want to overide in
+	 * run-hald.sh and friends)
+	 */
+	path = getenv ("PATH");
+	g_strlcpy (newpath, path, sizeof (newpath));
+	if (path != NULL) {
+		g_strlcat (newpath, ":", sizeof (newpath));
+		g_strlcat (newpath, PACKAGE_LIBEXEC_DIR, sizeof (newpath));
+	}
+	setenv ("PATH", newpath, TRUE);
+
 	while (1) {
 		int c;
 		int option_index = 0;
@@ -341,7 +434,7 @@ main (int argc, char *argv[])
 			{"daemon", 1, NULL, 0},
 			{"verbose", 1, NULL, 0},
 			{"help", 0, NULL, 0},
-			{"drop-privileges", 0, NULL, 0},
+			{"retain-privileges", 0, NULL, 0},
 			{NULL, 0, NULL, 0}
 		};
 
@@ -375,8 +468,9 @@ main (int argc, char *argv[])
 					usage ();
 					return 1;
 				}
-			} else if (strcmp (opt, "drop-privileges") == 0)
-				drop_privileges ();
+			} else if (strcmp (opt, "retain-privileges") == 0) {
+				retain_privs = TRUE;
+			}
 			break;
 
 		default:
@@ -391,6 +485,12 @@ main (int argc, char *argv[])
 	else
 		logger_disable ();
 
+	/* will fork into two; only the child will return here if we are successful */
+	/*master_slave_setup ();
+	  sleep (100000000);*/
+
+	loop = g_main_loop_new (NULL, FALSE);
+
 	HAL_INFO ((PACKAGE_STRING));
 	if (opt_become_daemon)
 		HAL_INFO (("Will daemonize"));
@@ -402,7 +502,7 @@ main (int argc, char *argv[])
 		int dev_null_fd;
 		int pf;
 		char pid[9];
-
+		
 		HAL_INFO (("Becoming a daemon"));
 
 		if (pipe (startup_daemonize_pipe) != 0) {
@@ -432,37 +532,31 @@ main (int argc, char *argv[])
 				dup2 (dev_null_fd, 0);
 				dup2 (dev_null_fd, 1);
 				dup2 (dev_null_fd, 2);
+				close (dev_null_fd);
 			}
 
 			umask (022);
 			break;
 
 		default:
-		        {
-				/* parent, block until child writes */
-				/* char buf[1];
-				read (startup_daemonize_pipe[0], &buf, sizeof (buf));*/
-				exit (0);
-				break;
-			}
+			/* parent, block until child writes */
+			exit (parent_wait_for_child (startup_daemonize_pipe[0], child_pid));
+			break;
 		}
 
 		/* Create session */
 		setsid ();
 
 		/* remove old pid file */
-		unlink(HALD_PID_FILE);
+		unlink (HALD_PID_FILE);
 
 		/* Make a new one */
-		if ((pf=open(HALD_PID_FILE, O_WRONLY|O_CREAT|O_TRUNC|O_EXCL,
-			0644)) == -1) {
-			HAL_ERROR (("Cannot create pid file"));
-			exit(1);
+		if ((pf= open (HALD_PID_FILE, O_WRONLY|O_CREAT|O_TRUNC|O_EXCL, 0644)) > 0) {
+			snprintf (pid, sizeof(pid), "%lu\n", (long unsigned) getpid ());
+			write (pf, pid, strlen(pid));
+			close (pf);
+			atexit (delete_pid);
 		}
-		sprintf(pid, "%lu\n", (long unsigned)getpid());
-		write(pf, pid, strlen(pid));
-		close(pf);
-		atexit(delete_pid);
 	}
 
 
@@ -488,22 +582,16 @@ main (int argc, char *argv[])
 	/* Finally, setup unix signal handler for TERM */
 	signal (SIGTERM, handle_sigterm);
 
-
-	hald_read_conf_file ();
-
-	/* set up the dbus services */
-	if (!hald_dbus_init ())
+	/* set up the local dbus server */
+	if (!hald_dbus_local_server_init ())
 		return 1;
 
-	loop = g_main_loop_new (NULL, FALSE);
-
-	/* initialize persitent property store, read uuid from path */
-	if (hald_get_conf ()->persistent_device_list)
-		hal_pstore_init (PACKAGE_LOCALSTATEDIR "/lib/hal/uuid");
-
 	/* initialize operating system specific parts */
 	osspec_init ();
 
+	if (!retain_privs)
+		drop_privileges();
+
 	hald_is_initialising = TRUE;
 
 	/* detect devices */
@@ -515,10 +603,31 @@ main (int argc, char *argv[])
 	return 0;
 }
 
-gboolean
-resolve_udiprop_path (const char *path, const char *source_udi,
-		      char *udi_result, size_t udi_result_size, 
-		      char *prop_result, size_t prop_result_size);
+#ifdef HALD_MEMLEAK_DBG
+extern int dbg_hal_device_object_delta;
+
+/* useful for valgrinding; see below */
+static gboolean
+my_shutdown (gpointer data)
+{
+	HalDeviceStore *gdl;
+	
+	printf ("Num devices in TDL: %d\n", g_slist_length ((hald_get_tdl ())->devices));
+	printf ("Num devices in GDL: %d\n", g_slist_length ((hald_get_gdl ())->devices));
+	
+	gdl = hald_get_gdl ();
+next:
+	if (g_slist_length (gdl->devices) > 0) {
+		HalDevice *d = HAL_DEVICE(gdl->devices->data);
+		hal_device_store_remove (gdl, d);
+		g_object_unref (d);
+		goto next;
+	}
+	
+	printf ("hal_device_object_delta = %d (should be zero)\n", dbg_hal_device_object_delta);
+	exit (1);
+}
+#endif
 
 void 
 osspec_probe_done (void)
@@ -527,27 +636,10 @@ osspec_probe_done (void)
 
 	HAL_INFO (("Device probing completed"));
 
-/*
-	{
-		char udi[256];
-		char prop[256];
-
-		resolve_udiprop_path ("info.udi", 
-				      "/org/freedesktop/Hal/devices/computer", 
-				      udi, sizeof (udi), 
-				      prop, sizeof (prop));
-		HAL_INFO (("----------------------------------------"));
-		resolve_udiprop_path ("/org/freedesktop/Hal/devices/computer:kernel.name", 
-				      "/org/freedesktop/Hal/devices/pci_8086_3341", 
-				      udi, sizeof (udi), 
-				      prop, sizeof (prop));
-		HAL_INFO (("----------------------------------------"));
-		resolve_udiprop_path ("@block.storage_device:@storage.physical_device:ide.channel", 
-				      "/org/freedesktop/Hal/devices/block_3_3", 
-				      udi, sizeof (udi), 
-				      prop, sizeof (prop));
+	if (!hald_dbus_init ()) {
+		hal_util_kill_all_helpers ();
+		exit (1);
 	}
-*/
 
 	/* tell parent to exit */
 	write (startup_daemonize_pipe[1], buf, sizeof (buf));
@@ -555,6 +647,12 @@ osspec_probe_done (void)
 	close (startup_daemonize_pipe[1]);
 
 	hald_is_initialising = FALSE;
+
+#ifdef HALD_MEMLEAK_DBG
+	g_timeout_add ((HALD_MEMLEAK_DBG) * 1000,
+		       my_shutdown,
+		       NULL);
+#endif
 }
 
 
