@@ -7,6 +7,7 @@ import dbus
 
 try:
     import gnome.ui
+
 except ImportError:
     gnome_imported = 0
 else:
@@ -35,33 +36,34 @@ class DeviceManager(LibGladeApplication):
         """Init the GUI and connect to the HAL daemon."""
         LibGladeApplication.__init__(self, Const.DATADIR + "/hal-device-manager.glade")
 
+        ver = getattr(dbus, 'version', (0, 0, 0))
+        if ver < (0, 40, 0):
+            dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, 
+                                       gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
+                                       "The DBus Python Bindings you are using are too old. "
+                                       "Make sure you have the latest version!")
+            dialog.run()
+            sys.exit(1)
+
         if not gnome_imported:
             self.xml.get_widget("about1").set_sensitive(0)
 
         self.representation = Representation()
 
-        self.bus = dbus.Bus(dbus.Bus.TYPE_SYSTEM)
-        self.hal_service = self.bus.get_service("org.freedesktop.Hal")
-        self.hal_manager = self.hal_service.get_object("/org/freedesktop/Hal/Manager",
-                                         "org.freedesktop.Hal.Manager")
+        self.bus = dbus.SystemBus()
+        self.hal_manager_obj = self.bus.get_object("org.freedesktop.Hal", 
+                                                   "/org/freedesktop/Hal/Manager")
+        self.hal_manager = dbus.Interface(self.hal_manager_obj,
+                                          "org.freedesktop.Hal.Manager")
 
         # gdl_changed will be invoked when the Global Device List is changed
         # per the hal spec
-        self.bus.add_signal_receiver(self.gdl_changed,
-				     "DeviceAdded",
-                                     "org.freedesktop.Hal.Manager",
-                                     "org.freedesktop.Hal",
-                                     "/org/freedesktop/Hal/Manager")
-        self.bus.add_signal_receiver(self.gdl_changed,
-				     "DeviceRemoved",
-                                     "org.freedesktop.Hal.Manager",
-                                     "org.freedesktop.Hal",
-                                     "/org/freedesktop/Hal/Manager")
-        self.bus.add_signal_receiver(self.gdl_changed,
-				     "NewCapability",
-                                     "org.freedesktop.Hal.Manager",
-                                     "org.freedesktop.Hal",
-                                     "/org/freedesktop/Hal/Manager")
+        self.hal_manager.connect_to_signal("DeviceAdded", 
+                         lambda *args: self.gdl_changed("DeviceAdded", *args))
+        self.hal_manager.connect_to_signal("DeviceRemoved", 
+                         lambda *args: self.gdl_changed("DeviceRemoved", *args))
+        self.hal_manager.connect_to_signal("NewCapability", 
+                         lambda *args: self.gdl_changed("NewCapability", *args))
 
         # Add listeners for all devices
         try:
@@ -82,26 +84,29 @@ class DeviceManager(LibGladeApplication):
         self.main_window.show()
 
     def add_device_signal_recv (self, udi):
-	self.bus.add_signal_receiver(self.device_changed,
+	self.bus.add_signal_receiver(lambda *args: self.property_modified(udi, *args),
 				     "PropertyModified",
 				     "org.freedesktop.Hal.Device",
 				     "org.freedesktop.Hal",
 				     udi)
 	return
-	self.bus.add_signal_receiver(self.device_changed,
+	self.bus.add_signal_receiver(lambda *args: self.device_condition(udi, *args),
 				     "Condition",
 				     "org.freedesktop.Hal.Device",
 				     "org.freedesktop.Hal",
 				     udi)
 
     def remove_device_signal_recv (self, udi):
-	self.bus.remove_signal_receiver(self.device_changed,
-				     "PropertyModified",
-				     "org.freedesktop.Hal.Device",
-				     "org.freedesktop.Hal",
-				     udi)
+	try:
+		self.bus.remove_signal_receiver(lambda *args: self.property_modified(udi, *args),
+					     "PropertyModified",
+					     "org.freedesktop.Hal.Device",
+					     "org.freedesktop.Hal",
+					     udi)
+	except:
+		print "FIXME: unknown reason for the error while removing this rule from dbus"
 	return
-	self.bus.remove_signal_receiver(self.device_changed,
+	self.bus.remove_signal_receiver(lambda *args: self.device_condition(udi, *args),
 				     "Condition",
 				     "org.freedesktop.Hal.Device",
 				     "org.freedesktop.Hal",
@@ -125,68 +130,63 @@ class DeviceManager(LibGladeApplication):
             self.update_device_notebook(device)
 
 
-    def device_changed(self, dbus_if, dbus_member, dbus_svc,
-                       dbus_obj_path, dbus_message):
+    def device_condition(self, device_udi, condition_name, condition_details):
         """This method is called when signals on the Device interface is
         received"""
-        if dbus_member=="Condition":
-            args = dbus_message.get_args_list()
-            print "\nCondition %s, device=%s"%(args[0], dbus_obj_path)
-            print "  message = ", args 
-        elif dbus_member=="PropertyModified":
-            args = dbus_message.get_args_list()
-            num_modifications = args[0]
-            print "\nPropertyModified, device=%s"%dbus_obj_path
-            #print "dbus_obj_path", dbus_obj_path
-            for i in range(0, num_modifications):
-                property_name = args[1+3*i]
-                removed = args[2+3*i]
-                added = args[3+3*i]
 
-                print "  key=%s, rem=%d, add=%d"%(property_name, removed, added)
-                if property_name=="info.parent":
-                    self.update_device_list()        
-                else:
-                    device_udi = dbus_obj_path
-                    device_udi_obj = self.hal_service.get_object(device_udi,
-                                                   "org.freedesktop.Hal.Device")
-                    device_obj = self.udi_to_device(device_udi)
+	print "\nCondition device=%s"%device_udi
+	print "  (condition_name, condition_details) = ('%s', '%s')"%(condition_name, condition_details)
 
-                    if device_udi_obj.PropertyExists(property_name):
-                        device_obj.properties[property_name] = device_udi_obj.GetProperty(property_name)
-                        print "  value=%s"%(device_obj.properties[property_name])
-                    else:
-                        if device_obj != None:
-                            try:
-                                del device_obj.properties[property_name]
-                            except:
-                                pass
+    def property_modified(self, device_udi, num_changes, change_list):
+        """This method is called when signals on the Device interface is
+        received"""
 
-                    device_focus_udi = self.get_current_focus_udi()
-                    if device_focus_udi != None:
-                        device = self.udi_to_device(device_udi)
-                        if device_focus_udi==device_udi:
-                            self.update_device_notebook(device)
+	print "\nPropertyModified, device=%s"%device_udi
+	for i in change_list:
+	    property_name = i[0]
+	    removed = i[1]
+	    added = i[2]
 
+	    print "  key=%s, rem=%d, add=%d"%(property_name, removed, added)
+	    if property_name=="info.parent":
+		self.update_device_list()        
+	    else:
+		device_udi_obj = self.bus.get_object("org.freedesktop.Hal", device_udi)
+		device_obj = self.udi_to_device(device_udi)
 
-    def gdl_changed(self, dbus_if, dbus_member, dbus_svc, dbus_obj_path, dbus_message):
+		if device_udi_obj.PropertyExists(property_name, dbus_interface="org.freedesktop.Hal.Device"):
+		    device_obj.properties[property_name] = device_udi_obj.GetProperty(property_name, 
+										      dbus_interface="org.freedesktop.Hal.Device")
+		    print "  value=%s"%(device_obj.properties[property_name])
+		else:
+		    if device_obj != None:
+			try:
+			    del device_obj.properties[property_name]
+			except:
+			    pass
+
+		device_focus_udi = self.get_current_focus_udi()
+		if device_focus_udi != None:
+		    device = self.udi_to_device(device_udi)
+		    if device_focus_udi==device_udi:
+			self.update_device_notebook(device)
+			
+    def gdl_changed(self, signal_name, device_udi, *args):
         """This method is called when a HAL device is added or removed."""
 
-        if dbus_member=="DeviceAdded":
-            [device_udi] = dbus_message.get_args_list()
+        if signal_name=="DeviceAdded":
             print "\nDeviceAdded, udi=%s"%(device_udi)
 	    self.add_device_signal_recv (device_udi)
             self.update_device_list()
-        elif dbus_member=="DeviceRemoved":
-            [device_udi] = dbus_message.get_args_list()
+        elif signal_name=="DeviceRemoved":
             print "\nDeviceRemoved, udi=%s"%(device_udi)
 	    self.remove_device_signal_recv (device_udi)
             self.update_device_list()
-        elif dbus_member=="NewCapability":
-            [device_udi, cap] = dbus_message.get_args_list()
+        elif signal_name=="NewCapability":
+            [cap] = args 
             print "\nNewCapability, cap=%s, udi=%s"%(cap, device_udi)
         else:
-            print "*** Unknown signal %s"%dbus_member
+            print "*** Unknown signal %s"% signal_name
 
 
     def update_device_list(self):
@@ -262,14 +262,16 @@ class DeviceManager(LibGladeApplication):
         
         # first build list of Device objects
         for name in device_names:
-            device_dbus_obj = self.hal_service.get_object(name,
-                                                     "org.freedesktop.Hal.Device")
-            properties = device_dbus_obj.GetAllProperties()
+            device_dbus_obj = self.bus.get_object("org.freedesktop.Hal" ,name)
+            properties = device_dbus_obj.GetAllProperties(dbus_interface="org.freedesktop.Hal.Device")
             try:
                 parent_name = properties["info.parent"]
             except KeyError:
                 # no parent, must be parent of virtual_root
                 parent_name = "/"
+            except TypeError:
+                print "Error: no properties for device %s"%name
+                continue
             device = Device(name, parent_name, properties)
             self.device_list.append(device)
 
@@ -298,12 +300,15 @@ class DeviceManager(LibGladeApplication):
         category = self.xml.get_widget("ns_device_category")
         capabilities = self.xml.get_widget("ns_device_capabilities")
 
-        # we always have Bus and State
-        bus.set_label(Const.BUS_NAMES[device.properties["info.bus"]])
+	if not device.properties.has_key("info.bus"):
+            product.set_label("Unknown")
+            vendor.set_label("Unknown")
+	else:
+	    bus.set_label(Const.BUS_NAMES[device.properties["info.bus"]])	    
         #state.set_label(Const.STATE_NAMES[device.properties["State"]])
 
         # guestimate product and vendor if we have no device information file
-        if device.properties["info.bus"]=="usb":
+        if device.properties.has_key("info.bus") and device.properties["info.bus"]=="usb":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             elif device.properties.has_key("usb.product"):
@@ -323,7 +328,7 @@ class DeviceManager(LibGladeApplication):
                 vendor.set_label("Unknown")
 
 
-        elif device.properties["info.bus"]=="pci":
+        elif device.properties.has_key("info.bus") and device.properties["info.bus"]=="pci":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             elif device.properties.has_key("pci.product"):
@@ -341,7 +346,7 @@ class DeviceManager(LibGladeApplication):
                 vendor.set_label("Unknown (0x%x)"%device.properties["pci.vendor_id"])
             else:
                 vendor.set_label("Unknown")
-        elif device.properties["info.bus"]=="block":
+        elif device.properties.has_key("info.bus") and device.properties["info.bus"]=="block":
             if device.properties.has_key("info.product"):
                 product.set_label("%s"%device.properties["info.product"])
             else:
@@ -370,7 +375,7 @@ class DeviceManager(LibGladeApplication):
     def update_tab_usb(self, device):
         """Updates the 'USB' tab given a Device object; may hide it"""
         page = self.xml.get_widget("device_notebook").get_nth_page(1)
-        if device.properties["info.bus"]!="usb":
+        if not device.properties.has_key("info.bus") or device.properties["info.bus"]!="usb":
             page.hide_all()
             return
 
@@ -404,7 +409,7 @@ class DeviceManager(LibGladeApplication):
     def update_tab_pci(self, device):
         """Updates the 'PCI' tab given a Device object; may hide it"""
         page = self.xml.get_widget("device_notebook").get_nth_page(2)
-        if device.properties["info.bus"]!="pci":
+        if not device.properties.has_key("info.bus") or device.properties["info.bus"]!="pci":
             page.hide_all()
             return
 
@@ -457,6 +462,9 @@ class DeviceManager(LibGladeApplication):
                     store.set(iter, 0, p, 1, "bool", 2, "false")
             elif ptype==float:
                 store.set(iter, 0, p, 1, "float", 2, "%f"%val)
+	    else:
+		# assume strlist
+		store.set(iter, 0, p, 1, "strlist", 2, val)
 
 
         prop_tree_view = self.xml.get_widget("ns_adv_properties")
