@@ -11,7 +11,7 @@ class Representation:
     def load_and_scale_icon(self, path):
         """Helper function for loading an icon and scaling it to 16x16"""
         orig = gtk.gdk.pixbuf_new_from_file(path)
-        icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, gtk.TRUE, 8, 16, 16)
+        icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 16, 16)
         orig.scale(icon, 0, 0, 16, 16,
                    0, 0,
                    16.0/orig.get_width(),
@@ -47,16 +47,25 @@ class Representation:
         # Default to abstract icon
         icon = self.icons["abstract"]
 
-        product = device.properties.get("info.product", "")
+        try:
+            product = device.properties["info.product"]
+        except KeyError:
+            product = "Unknown"
+        except TypeError:
+            return icon
+            
         if product=="Computer":
             return self.icons["computer"]
 
         # First look at bus type, every device got Bus property
-        bus = device.properties["info.bus"]
-        if bus=="usb_device":
-            icon = self.icons["bus_usb"]
-        elif bus=="pci":
-            icon = self.icons["bus_pci"]
+        if device.properties.has_key("info.bus"):
+	    bus = device.properties["info.bus"]
+	    if bus=="usb_device":
+		icon = self.icons["bus_usb"]
+	    elif bus=="pci":
+		icon = self.icons["bus_pci"]
+	else:
+	    bus = "unknown"
 
         # Then look at Category, if available
         if not device.properties.has_key("info.category"):
