@@ -60,8 +60,8 @@
 
 
 #define DBUS_API_SUBJECT_TO_CHANGE
-#include "libhal/libhal.h"
-#include "libhal-storage/libhal-storage.h"
+#include "../libhal/libhal.h"
+#include "../libhal-storage/libhal-storage.h"
 
 typedef int boolean;
 
@@ -888,9 +888,11 @@ fs_table_has_device (FSTable *table, const char *block_device, const char *label
 		    }
 
 		    if (strcmp (block_device, buf) == 0) {
-		      /* update block.device with new value */
-		      fstab_update_debug (_("%d: Found %s pointing to %s in" _PATH_FSTAB), pid, field->value, buf);
-		      hal_device_set_property_string (hal_context, udi, "block.device", field->value);
+			    DBusError error;
+			    /* update block.device with new value */
+			    fstab_update_debug (_("%d: Found %s pointing to %s in" _PATH_FSTAB), pid, field->value, buf);
+			    dbus_error_init (&error);
+			    libhal_device_set_property_string (hal_context, udi, "block.device", field->value, &error);
 		      return TRUE;
 		    }
 
@@ -1144,8 +1146,8 @@ tryagain:
 static char* add_hal_device (FSTable *table, const char *udi)
 {
 	char *rc;
-	HalDrive *drive;
-	HalVolume *volume;
+	LibHalDrive *drive;
+	LibHalVolume *volume;
 	FSTableLine *line;
 	char final_options[256];
 	
@@ -1155,21 +1157,24 @@ static char* add_hal_device (FSTable *table, const char *udi)
 	
 	fstab_update_debug (_("%d: entering add_hal_device, udi='%s'\n"), pid, udi);
 	
-	drive  = hal_drive_from_udi (hal_context, udi);
-	/*fstab_update_debug (_("%d: drive=%x, volume=%x\n"), pid, drive, volume);*/
+	drive  = libhal_drive_from_udi (hal_context, udi);
 	if (drive == NULL) {
 		char *udi_storage_device;
+		DBusError error;
 		/* try block.storage_device */
-		udi_storage_device = hal_device_get_property_string (hal_context, udi, "block.storage_device");
+		dbus_error_init (&error);
+		udi_storage_device = libhal_device_get_property_string (hal_context, udi, "block.storage_device", &error);
 		if (udi_storage_device == NULL)
 			goto out;
-		drive = hal_drive_from_udi (hal_context, udi_storage_device);
-		hal_free_string (udi_storage_device);
+		drive = libhal_drive_from_udi (hal_context, udi_storage_device);
+		libhal_free_string (udi_storage_device);
 		if (drive == NULL)
 			goto out;
 	}
 
-	volume = hal_volume_from_udi (hal_context, udi);
+	volume = libhal_volume_from_udi (hal_context, udi);
+
+	/*fstab_update_debug (_("%d: drive=%x, volume=%x\n"), pid, drive, volume);*/
 		
 	/* see if we are a drive */
 	if (volume == NULL) {
@@ -1180,20 +1185,22 @@ static char* add_hal_device (FSTable *table, const char *udi)
 		const char *fstype;
 		const char *options;
 		
-		if (!hal_drive_policy_is_mountable (drive, NULL))
+		if (!libhal_drive_policy_is_mountable (drive, NULL))
 			goto out;
 		
-		device_file = hal_drive_get_device_file (drive);
+		device_file = libhal_drive_get_device_file (drive);
 		if (device_file == NULL)
 			goto out;
 
+		fstab_update_debug (_("%d: device_file=%s\n"), pid, device_file);
+
 		if (fs_table_has_device (table, device_file, NULL, udi)) /* no label */ {
 			fstab_update_debug (_("%d: Could not add entry to fstab file: "
 					      "block device %s already listed\n"), pid, device_file);
 			goto out;
 		}
 		
-		desired_mount_point = hal_drive_policy_get_desired_mount_point (drive, NULL);
+		desired_mount_point = libhal_drive_policy_get_desired_mount_point (drive, NULL);
 		if (desired_mount_point == NULL)
 			goto out;
 
@@ -1201,10 +1208,10 @@ static char* add_hal_device (FSTable *table, const char *udi)
 		if (normalized_desired_mount_point == NULL)
 			goto out;
 		
-		fstype = hal_drive_policy_get_mount_fs (drive, NULL);
+		fstype = libhal_drive_policy_get_mount_fs (drive, NULL);
 		if (fstype == NULL)
 			goto out;
-		options = hal_drive_policy_get_mount_options (drive, NULL);
+		options = libhal_drive_policy_get_mount_options (drive, NULL);
 		if (options == NULL)
 			goto out;
 		
@@ -1243,14 +1250,14 @@ static char* add_hal_device (FSTable *table, const char *udi)
 		const char *options;
 		
 		/* means we are a volume */
-		if (!hal_volume_policy_is_mountable (drive, volume, NULL))
+		if (!libhal_volume_policy_is_mountable (drive, volume, NULL))
 			goto out;
 		
-		device_file = hal_volume_get_device_file (volume);
+		device_file = libhal_volume_get_device_file (volume);
 		if (device_file == NULL)
 			goto out;
 		
-		label = hal_volume_get_label (volume);
+		label = libhal_volume_get_label (volume);
 		if (device_file == NULL)
 			goto out;
 		
@@ -1260,7 +1267,7 @@ static char* add_hal_device (FSTable *table, const char *udi)
 			goto out;
 		}
 		
-		desired_mount_point = hal_volume_policy_get_desired_mount_point (drive, volume, NULL);
+		desired_mount_point = libhal_volume_policy_get_desired_mount_point (drive, volume, NULL);
 		if (desired_mount_point == NULL)
 			goto out;
 
@@ -1268,10 +1275,10 @@ static char* add_hal_device (FSTable *table, const char *udi)
 		if (normalized_desired_mount_point == NULL)
 			goto out;
 		
-		fstype = hal_volume_policy_get_mount_fs (drive, volume, NULL);
+		fstype = libhal_volume_policy_get_mount_fs (drive, volume, NULL);
 		if (fstype == NULL)
 			goto out;
-		options = hal_volume_policy_get_mount_options (drive, volume, NULL);
+		options = libhal_volume_policy_get_mount_options (drive, volume, NULL);
 		if (options == NULL)
 			goto out;
 		
@@ -1306,8 +1313,8 @@ static char* add_hal_device (FSTable *table, const char *udi)
 	
 	
 out:
-	hal_volume_free (volume);
-	hal_drive_free (drive);
+	libhal_volume_free (volume);
+	libhal_drive_free (drive);
 	return rc;
 }
 
@@ -1323,6 +1330,7 @@ add_udi (const char *udi)
 	char *last_slash;
 	char *mount_point;
 	char *device_file;
+	DBusError error;
 	
 	dir = strdup (_PATH_FSTAB); 	 
 	last_slash = strrchr (dir, '/'); 	 
@@ -1370,14 +1378,15 @@ add_udi (const char *udi)
 	if (!create_mount_point_for_volume (mount_point))
 		goto error;
 
-	device_file = hal_device_get_property_string (hal_context, udi, "block.device");
+	dbus_error_init (&error);
+	device_file = libhal_device_get_property_string (hal_context, udi, "block.device", &error);
 	
 	fstab_update_debug (_("%d: added mount point '%s' for device '%s'\n"),
 			    pid, mount_point, device_file);
 	syslog (LOG_INFO, _("added mount point %s for %s"), 
 		mount_point, device_file);
 	
-	hal_free_string (device_file);
+	libhal_free_string (device_file);
 	
 	close (fd);
 	
@@ -1405,20 +1414,22 @@ remove_udi (const char *udi)
   boolean is_volume;
   char *dir = NULL;
   char *last_slash;
+  DBusError error;
 
-  is_volume = hal_device_query_capability (hal_context, udi, "volume");
+  dbus_error_init (&error);
+  is_volume = libhal_device_query_capability (hal_context, udi, "volume", &error);
 
   /* don't remove the fstab entry if we were spawned of a device with
    * storage.no_partitions_hint set to TRUE. Per the spec this is
    * exactly when block.no_partitions is TRUE on the volume. E.g.
    * floppies and optical discs
    */
-  if (is_volume && hal_device_get_property_bool (hal_context, udi, "block.no_partitions"))
+  dbus_error_init (&error);
+  if (is_volume && libhal_device_get_property_bool (hal_context, udi, "block.no_partitions", &error))
     return FALSE;
 
-  block_device = hal_device_get_property_string (hal_context, udi, "block.device");
-  if (block_device == NULL)
-	  return FALSE;
+  dbus_error_init (&error);
+  block_device = libhal_device_get_property_string (hal_context, udi, "block.device", &error);
 
   dir = strdup (_PATH_FSTAB); 	 
   last_slash = strrchr (dir, '/'); 	 
@@ -1655,6 +1666,8 @@ main (int argc, const char *argv[])
   char *udi_to_add = NULL, *udi_to_remove = NULL, *hal_device_udi;
   const char **left_over_args = NULL;
   int lockfd = -1;
+  DBusError error;
+  DBusConnection *conn;
 
   pid = getpid ();
 
@@ -1693,7 +1706,9 @@ main (int argc, const char *argv[])
   hal_device_udi = getenv ("UDI");
   if (hal_device_udi != NULL) {
       char *caps;
-      
+      char *action;
+     
+ 
       /* when invoked for the /org/freedesktop/Hal/devices/computer UDI we clean the fstab */
       if (getenv ("HALD_STARTUP") != NULL && strcmp (hal_device_udi, "/org/freedesktop/Hal/devices/computer") == 0) {
 	should_clean = TRUE;
@@ -1720,6 +1735,13 @@ main (int argc, const char *argv[])
 	  }
 	}
 
+	if ((action = getenv ("HALD_ACTION")) != NULL) {
+		if (strcmp (action, "add") == 0)
+			udi_to_add = hal_device_udi;
+		else if (strcmp (action, "remove") == 0)
+			udi_to_remove = hal_device_udi;
+	}
+
 	/* we don't want to remove entries just because hald is shutting down */
 	if (getenv ("HALD_SHUTDOWN") != NULL)
 	  goto out;
@@ -1765,17 +1787,22 @@ main (int argc, const char *argv[])
 		  }
 	  }
 
-  hal_context = hal_initialize (NULL, FALSE);
-  fsy_mount_root = hal_drive_policy_default_get_mount_root (hal_context);
+  dbus_error_init (&error);	
+  if ((hal_context = libhal_ctx_init_direct (&error)) == NULL) {
+	  fprintf (stderr, "error: libhal_ctx_init_direct: %s: %s\n", error.name, error.message);
+	  goto out;
+  }
+
+  fsy_mount_root = libhal_drive_policy_default_get_mount_root (hal_context);
   if (fsy_mount_root == NULL)
 	  goto out;
   fstab_update_debug (_("%d: mount_root='%s'\n"), pid, fsy_mount_root);
-  fsy_use_managed = hal_drive_policy_default_use_managed_keyword (hal_context);
+  fsy_use_managed = libhal_drive_policy_default_use_managed_keyword (hal_context);
   fstab_update_debug (_("%d: use_managed=%d\n"), pid, fsy_use_managed);
   if (fsy_use_managed) {
-	  if ((fsy_managed_primary   = hal_drive_policy_default_get_managed_keyword_primary (hal_context)) == NULL)
+	  if ((fsy_managed_primary   = libhal_drive_policy_default_get_managed_keyword_primary (hal_context)) == NULL)
 		  goto out;
-	  if ((fsy_managed_secondary = hal_drive_policy_default_get_managed_keyword_secondary (hal_context)) == NULL)
+	  if ((fsy_managed_secondary = libhal_drive_policy_default_get_managed_keyword_secondary (hal_context)) == NULL)
 		  goto out;
 	  fstab_update_debug (_("%d: managed primary='%s'\n"), pid, fsy_managed_primary);
 	  fstab_update_debug (_("%d: managed secondary='%s'\n"), pid, fsy_managed_secondary);
@@ -1789,7 +1816,8 @@ main (int argc, const char *argv[])
       if (udi_to_remove)
         retval |= !remove_udi (udi_to_remove);
 
-      hal_shutdown (hal_context);
+      libhal_ctx_shutdown (hal_context, &error);
+      libhal_ctx_free (hal_context);
     }
   else if (should_clean)
     retval = clean ();
